@@ -1,5 +1,5 @@
 // rak - Rakshasa's toolbox
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ public:
   static const int e_noent       = ENOENT;
   static const int e_nomem       = ENOMEM;
   static const int e_notdir      = ENOTDIR;
+  static const int e_isdir       = EISDIR;
   
   static const int e_intr        = EINTR;
 
@@ -62,7 +63,7 @@ public:
   bool                is_valid() const             { return m_errno != 0; }
 
   int                 value() const                { return m_errno; }
-  const char*         c_str() const                { return strerror(m_errno); }
+  const char*         c_str() const                { return std::strerror(m_errno); }
 
   bool                is_blocked_momentary() const { return m_errno == e_again || m_errno == e_intr; }
   bool                is_blocked_prolonged() const { return m_errno == e_deadlk; }
@@ -73,6 +74,7 @@ public:
 
   static error_number current()                    { return errno; }
   static void         clear_global()               { errno = 0; }
+  static void         set_global(error_number err) { errno = err.m_errno; }
 
   bool operator == (const error_number& e) const   { return m_errno == e.m_errno; }
 
