@@ -37,43 +37,45 @@
 #ifndef RAK_PRIORITY_QUEUE_DEFAULT_H
 #define RAK_PRIORITY_QUEUE_DEFAULT_H
 
-#include <stdexcept>
-#include <rak/functional.h>
-#include <rak/functional_fun.h>
+#include <tr1/functional>
+#include <rak/allocators.h>
 #include <rak/priority_queue.h>
 #include <rak/timer.h>
 
+#include "torrent/exceptions.h"
+
 namespace rak {
 
 class priority_item {
 public:
+  typedef std::tr1::function<void (void)> slot_void;
+
   priority_item() {}
   ~priority_item() {
     if (is_queued())
-      throw std::logic_error("priority_item::~priority_item() called on a queued item.");
+      throw torrent::internal_error("priority_item::~priority_item() called on a queued item.");
 
     m_time = timer();
-    m_slot.set(NULL);
+    m_slot = slot_void();
   }
 
-  bool                is_valid() const                        { return m_slot.is_valid(); }
-  bool                is_queued() const                       { return m_time != timer(); }
+  bool                is_valid() const              { return (bool)m_slot; }
+  bool                is_queued() const             { return m_time != timer(); }
 
-  void                call()                                  { m_slot(); }
-  void                set_slot(function0<void>::base_type* s) { m_slot.set(s); }
-  
-  const timer&        time() const                           { return m_time; }
-  void                clear_time()                           { m_time = timer(); }
-  void                set_time(const timer& t)               { m_time = t; }
+  slot_void&          slot()                        { return m_slot; }
 
-  bool                compare(const timer& t) const          { return m_time > t; }
+  const timer&        time() const                  { return m_time; }
+  void                clear_time()                  { m_time = timer(); }
+  void                set_time(const timer& t)      { m_time = t; }
+
+  bool                compare(const timer& t) const { return m_time > t; }
 
 private:
   priority_item(const priority_item&);
   void operator = (const priority_item&);
 
   timer               m_time;
-  function0<void>     m_slot;
+  slot_void           m_slot;
 };
 
 struct priority_compare {
@@ -83,7 +85,8 @@ struct priority_compare {
 };
 
 typedef std::equal_to<priority_item*> priority_equal;
-typedef priority_queue<priority_item*, priority_compare, priority_equal> priority_queue_default;
+typedef priority_queue<priority_item*, priority_compare, priority_equal,
+                       cacheline_allocator<priority_item*> > priority_queue_default;
 
 inline void
 priority_queue_perform(priority_queue_default* queue, timer t) {
@@ -92,23 +95,23 @@ priority_queue_perform(priority_queue_default* queue, timer t) {
     queue->pop();
 
     v->clear_time();
-    v->call();
+    v->slot()();
   }
 }
 
 inline void
 priority_queue_insert(priority_queue_default* queue, priority_item* item, timer t) {
   if (t == timer())
-    throw std::logic_error("priority_queue_insert(...) received a bad timer.");
+    throw torrent::internal_error("priority_queue_insert(...) received a bad timer.");
 
   if (!item->is_valid())
-    throw std::logic_error("priority_queue_insert(...) called on an invalid item.");
+    throw torrent::internal_error("priority_queue_insert(...) called on an invalid item.");
 
   if (item->is_queued())
-    throw std::logic_error("priority_queue_insert(...) called on an already queued item.");
+    throw torrent::internal_error("priority_queue_insert(...) called on an already queued item.");
 
   if (queue->find(item) != queue->end())
-    throw std::logic_error("priority_queue_insert(...) item found in queue.");
+    throw torrent::internal_error("priority_queue_insert(...) item found in queue.");
 
   item->set_time(t);
   queue->push(item);
@@ -122,16 +125,16 @@ priority_queue_erase(priority_queue_default* queue, priority_item* item) {
   // Check is_valid() after is_queued() so that it is safe to call
   // erase on untouched instances.
   if (!item->is_valid())
-    throw std::logic_error("priority_queue_erase(...) called on an invalid item.");
+    throw torrent::internal_error("priority_queue_erase(...) called on an invalid item.");
 
   // Clear time before erasing to force it to the top.
   item->clear_time();
   
   if (!queue->erase(item))
-    throw std::logic_error("priority_queue_erase(...) could not find item in queue.");
+    throw torrent::internal_error("priority_queue_erase(...) could not find item in queue.");
 
   if (queue->find(item) != queue->end())
-    throw std::logic_error("priority_queue_erase(...) item still in queue.");
+    throw torrent::internal_error("priority_queue_erase(...) item still in queue.");
 }
 
 }
