@@ -238,7 +238,7 @@ copy_escape_html(Iterator first, Iterator last) {
 }
 
 template <typename Sequence>
-Sequence
+inline Sequence
 copy_escape_html(const Sequence& src) {
   Sequence dest;
   copy_escape_html(src.begin(), src.end(), std::back_inserter(dest));
@@ -246,6 +246,15 @@ copy_escape_html(const Sequence& src) {
   return dest;
 }
 
+template <typename Sequence>
+inline std::string
+copy_escape_html_str(const Sequence& src) {
+  std::string dest;
+  copy_escape_html(src.begin(), src.end(), std::back_inserter(dest));
+
+  return dest;
+}
+
 // Consider support for larger than char type.
 template <typename InputIterator, typename OutputIterator> 
 OutputIterator
@@ -291,6 +300,15 @@ transform_hex(Iterator first, Iterator last) {
   return dest;
 }
 
+template <typename Sequence>
+inline std::string
+transform_hex_str(const Sequence& seq) {
+  std::string dest;
+  transform_hex(seq.begin(), seq.end(), std::back_inserter(dest));
+
+  return dest;
+}
+
 template <typename Sequence>
 Sequence
 generate_random(size_t length) {
