@@ -36,6 +36,7 @@
 
 #include "config.h"
 
+#include <rak/functional.h>
 #include <rak/timer.h> 
 #include <rak/priority_queue_default.h> 
 
@@ -61,7 +62,7 @@ ThrottleInternal::ThrottleInternal(int flags) :
   m_timeLastTick(cachedTime) {
 
   if (is_root())
-    m_taskTick.set_slot(rak::mem_fn(this, &ThrottleInternal::receive_tick));
+    m_taskTick.slot() = std::tr1::bind(&ThrottleInternal::receive_tick, this);
 }
 
 ThrottleInternal::~ThrottleInternal() {
