@@ -154,6 +154,9 @@ Handshake::initialize_outgoing(const rak::socket_address& sa, DownloadMain* d, P
 
 void
 Handshake::deactivate_connection() {
+  if (!get_fd().is_valid())
+    throw internal_error("Handshake::deactivate_connection called but m_fd is not open.");
+
   m_state = INACTIVE;
 
   priority_queue_erase(&taskScheduler, &m_taskTimeout);
@@ -166,6 +169,9 @@ Handshake::deactivate_connection() {
 
 void
 Handshake::release_connection() {
+  if (!get_fd().is_valid())
+    throw internal_error("Handshake::release_connection called but m_fd is not open.");
+
   m_peerInfo->unset_flags(PeerInfo::flag_handshake);
   m_peerInfo = NULL;
 
@@ -174,6 +180,9 @@ Handshake::release_connection() {
 
 void
 Handshake::destroy_connection() {
+  if (!get_fd().is_valid())
+    throw internal_error("Handshake::destroy_connection called but m_fd is not open.");
+
   manager->connection_manager()->dec_socket_count();
 
   get_fd().close();
@@ -280,7 +289,8 @@ Handshake::read_encryption_key() {
   if (m_incoming)
     prepare_key_plus_pad();
 
-  m_encryption.key()->compute_secret(m_readBuffer.position(), 96);
+  if(!m_encryption.key()->compute_secret(m_readBuffer.position(), 96))
+    throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_encryption);
   m_readBuffer.consume(96);
 
   // Determine the synchronisation string.
@@ -979,7 +989,8 @@ Handshake::prepare_proxy_connect() {
 
 void
 Handshake::prepare_key_plus_pad() {
-  m_encryption.initialize();
+  if (!m_encryption.initialize())
+    throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
 
   m_encryption.key()->store_pub_key(m_writeBuffer.end(), 96);
   m_writeBuffer.move_end(96);
@@ -1068,9 +1079,12 @@ Handshake::prepare_peer_info() {
   }
 
   std::memcpy(m_peerInfo->set_options(), m_options, 8);
+
   m_peerInfo->mutable_id().assign((const char*)m_readBuffer.position());
   m_readBuffer.consume(20);
 
+  hash_string_to_hex(m_peerInfo->id(), m_peerInfo->mutable_id_hex());
+
   // For meta downloads, we require support of the extension protocol.
   if (m_download->info()->is_meta_download() && !m_peerInfo->supports_extensions())
     throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unwanted_connection);
