@@ -39,8 +39,6 @@
 #define __STDC_FORMAT_MACROS
 
 #include <inttypes.h>
-#include <sigc++/adaptors/bind.h>
-#include <sigc++/adaptors/hide.h>
 
 #include "data/block.h"
 #include "data/block_list.h"
@@ -169,10 +167,11 @@ Download::stop(int flags) {
   LT_LOG_THIS(INFO, "Stopping torrent: flags:%0x.", flags);
 
   m_ptr->main()->stop();
-  m_ptr->main()->tracker_controller()->disable();
 
   if (!(flags & stop_skip_tracker))
     m_ptr->main()->tracker_controller()->send_stop_event();
+
+  m_ptr->main()->tracker_controller()->disable();
 }
 
 bool
