@@ -48,7 +48,7 @@
 #include "torrent/exceptions.h"
 #include "download/download_main.h"
 
-namespace std { using namespace tr1; }
+namespace tr1 { using namespace std::tr1; }
 
 namespace torrent {
 
@@ -61,14 +61,14 @@ uint64_t
 choke_group::up_rate() const {
   return
     std::for_each(m_first, m_last, 
-                  rak::accumulate((uint64_t)0, std::bind(&Rate::rate, std::bind(&resource_manager_entry::up_rate, std::placeholders::_1)))).result;
+                  rak::accumulate((uint64_t)0, tr1::bind(&Rate::rate, tr1::bind(&resource_manager_entry::up_rate, tr1::placeholders::_1)))).result;
 }
 
 uint64_t
 choke_group::down_rate() const {
   return
     std::for_each(m_first, m_last, 
-                  rak::accumulate((uint64_t)0, std::bind(&Rate::rate, std::bind(&resource_manager_entry::down_rate, std::placeholders::_1)))).result;
+                  rak::accumulate((uint64_t)0, tr1::bind(&Rate::rate, tr1::bind(&resource_manager_entry::down_rate, tr1::placeholders::_1)))).result;
 }
 
 }
