@@ -70,7 +70,16 @@ DiffieHellman::~DiffieHellman() {
 #endif
 };
 
-void
+bool
+DiffieHellman::is_valid() const {
+#ifdef USE_OPENSSL
+  return m_dh != NULL && m_dh->pub_key != NULL;
+#else
+  return false;
+#endif
+}
+
+bool
 DiffieHellman::compute_secret(const unsigned char *pubkey, unsigned int length) {
 #ifdef USE_OPENSSL
   BIGNUM* k = BN_bin2bn(pubkey, length, NULL);
@@ -81,6 +90,10 @@ DiffieHellman::compute_secret(const unsigned char *pubkey, unsigned int length)
   m_size = DH_compute_key((unsigned char*)m_secret, k, m_dh);
   
   BN_free(k);
+
+  return m_size != -1;
+#else
+  return false;
 #endif
 };
 
