@@ -17,7 +17,7 @@
 
 CPPUNIT_TEST_SUITE_REGISTRATION(option_strings_test);
 
-namespace std { using namespace tr1; }
+namespace tr1 { using namespace std::tr1; }
 
 void
 option_strings_test::test_basic() {
